# charset = utf-8

from django.http import JsonResponse
from neomodel import db

from block.models import Account, Block, Contract, Transaction, Miner, gen_node_json, gen_rel_json, gen_node_with_properties_json
from utils import tools


def get_account(account, from_time, end_time):
    from_time = tools.from_str_to_time(from_time)
    end_time = tools.from_str_to_time(end_time)

    addresses = []
    if account:
        if from_time:
            param = {"account": account, 'from_time': from_time.timestamp(), "end_time": end_time.timestamp()}
            query = 'match(a:Account)-[out:TRANSFER_OUT]->(t:Transaction) ' \
                    'where a.address contains $account and out.date >= $from_time and out.date <= $end_time return a.address limit 100'
            items = db.cypher_query(query, param)[0]
            addresses = [item[0] for item in items]
        else:
            nodes = Account.nodes.filter(address__contains=account).all()
            if nodes:
                addresses = [node.address for node in nodes]
    else:
        if from_time:
            param = {'from_time': from_time.timestamp(), "end_time": end_time.timestamp()}
            query = 'match(a:Account)-[out:TRANSFER_OUT]->(t:Transaction) ' \
                    'where out.date >= $from_time and out.date <= $end_time return a.address limit 100'
            items = db.cypher_query(query, param)[0]
            addresses = [item[0] for item in items]
        else:
            nodes = Account.nodes[:100]
            addresses = [node.address for node in nodes]
    return addresses


def get_block_list(request):
    if request.method == "GET":
        try:
            account = request.GET.get("account")
            from_time = request.GET.get('from_time')
            end_time = request.GET.get('end_time')
            sort_type = request.GET.get('sort_type')

            if from_time and not end_time:
                resp = tools.dec_error_resp("end_time is null")
                return JsonResponse(resp)
            if not from_time and end_time:
                resp = tools.dec_error_resp("from_time is null")
                return JsonResponse(resp)

            resp_nodes, resp_relations = [], []
            node_ids, rel_ids = [], []
            addresses = get_account(account, from_time, end_time)
            if len(addresses) == 0:
                resp = tools.dec_success_resp({"nodes": resp_nodes, "relations": resp_relations})
                return JsonResponse(resp, safe=False)

            accounts = Account.nodes.filter(address__in=addresses)

            param = {'addresses': addresses}
            if sort_type and int(sort_type) != 2:
                query = 'match(a:Account)-[out:TRANSFER_OUT]->(t:Transaction) where a.address in $addresses ' \
                        'return a.address, count(out) order by count(out) desc'
            else:
                query = 'match(a:Account)-[out:TRANSFER_OUT]->(t:Transaction) where a.address in $addresses ' \
                        'return a.address, sum(t.amount) order by sum(t.amount) desc'
            items = db.cypher_query(query, param)[0]
            max_address = items[0][0]
            max_num = items[0][1]
            min_num = items[len(items)-1][1]

            deal_dict = {}
            for item in items:
                address, num = item
                deal_dict[address] = num

            max_id = 0
            for a in accounts:
                if a.address == max_address:
                    max_id = a.id
                    break

            # 账户
            transactions = []
            for a in accounts:
                if sort_type and int(sort_type) != 2:
                    resp_nodes.append(gen_node_with_properties_json(a, {"dealNum": deal_dict.get(a.address), "maxNum": max_num, "minNum": min_num}, max_id))
                else:
                    resp_nodes.append(gen_node_with_properties_json(a, {"dealAmount": deal_dict.get(a.address), "maxAmount": max_num, "minAmount": min_num}, max_id))
                node_ids.append(a.id)
                next_nodes = a.transfer_out.all()
                for next_node in next_nodes:
                    if next_node.id not in node_ids:
                        transactions.append(next_node)
                        node_ids.append(next_node.id)
                    for r in a.transfer_out.all_relationships(next_node):
                        if r.id not in rel_ids:
                            resp_relations.append(gen_rel_json(r))
                            rel_ids.append(r.id)

            # 交易
            blocks = []
            for t in transactions:
                resp_nodes.append(gen_node_json(t))
                node_ids.append(t.id)
                next_account_nodes = t.transfer_in.all()
                for next_node in next_account_nodes:
                    if next_node.id not in node_ids:
                        resp_nodes.append(gen_node_json(next_node))
                        node_ids.append(next_node.id)
                    for r in t.transfer_in.all_relationships(next_node):
                        if r.id not in rel_ids:
                            resp_relations.append(gen_rel_json(r))
                            rel_ids.append(r.id)
                next_contract_nodes = t.invoke.all()
                for next_node in next_contract_nodes:
                    if next_node.id not in node_ids:
                        resp_nodes.append(gen_node_json(next_node))
                        node_ids.append(next_node.id)
                    for r in t.invoke.all_relationships(next_node):
                        if r.id not in rel_ids:
                            resp_relations.append(gen_rel_json(r))
                            rel_ids.append(r.id)
                next_block_nodes = t.trade.all()
                for next_node in next_block_nodes:
                    if next_node.id not in node_ids:
                        blocks.append(next_node)
                    for r in t.trade.all_relationships(next_node):
                        if r.id not in rel_ids:
                            resp_relations.append(gen_rel_json(r))
                            rel_ids.append(r.id)

            # 区块
            block_hashs = []
            while blocks:
                tmp_nodes = []
                for node in blocks:
                    block_hashs.append(node.hash)
                    if node.id not in node_ids:
                        resp_nodes.append(gen_node_json(node))
                        node_ids.append(node.id)
                        next_nodes = node.next_block.all()
                        for next_node in next_nodes:
                            if next_node.id not in node_ids:
                                tmp_nodes.append(next_node)
                            for r in node.next_block.all_relationships(next_node):
                                if r.id not in rel_ids:
                                    resp_relations.append(gen_rel_json(r))
                                    rel_ids.append(r.id)
                blocks = tmp_nodes
            block_hashs = list(set(block_hashs))

            # 矿工
            param = {'hashes': block_hashs}
            query = 'match (m:Miner)-[p:PACK]->(b:Block) where b.hash in $hashes return m.address, p, b.hash'
            items = db.cypher_query(query, param)[0]

            for item in items:
                m_address, p, b_hash = item
                m = Miner.nodes.get_or_none(address=m_address)
                b = Block.nodes.get_or_none(hash=b_hash)
                if m and m.id not in node_ids:
                    node_ids.append(m.id)
                    resp_nodes.append(gen_node_json(m))
                if p and p.id not in rel_ids:
                    resp_relations.append({"id": p.id, "source": m.id, "target": b.id, "label": "打包"})
                    rel_ids.append(p.id)

            resp_nodes.sort(key=lambda n: n.get("id"))
            resp_relations.sort(key=lambda r: r.get("id"))
            resp = tools.dec_success_resp({"nodes": resp_nodes, "relations": resp_relations})
            return JsonResponse(resp, safe=False)

        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)


def get_block_trade_rank(request):
    if request.method == "GET":
        try:
            account = request.GET.get("account")
            from_time = request.GET.get('from_time')
            end_time = request.GET.get('end_time')
            sort_type = request.GET.get("sort_type")

            if from_time and not end_time:
                resp = tools.dec_error_resp("end_time is null")
                return JsonResponse(resp)
            if not from_time and end_time:
                resp = tools.dec_error_resp("from_time is null")
                return JsonResponse(resp)

            rank = []
            addresses = get_account(account, from_time, end_time)
            if len(addresses) == 0:
                resp = tools.dec_success_resp({"count": 0, "rank": rank})
                return JsonResponse(resp, safe=False)

            param = {'addresses': addresses}
            if sort_type and int(sort_type) != 2:
                query = 'match(a:Account)-[out:TRANSFER_OUT]->(t:Transaction) where a.address in $addresses ' \
                        'return a.address, count(out), sum(t.amount) order by count(out) desc'
            else:
                query = 'match(a:Account)-[out:TRANSFER_OUT]->(t:Transaction) where a.address in $addresses ' \
                        'return a.address, count(out), sum(t.amount) order by sum(t.amount) desc'
            items = db.cypher_query(query, param)[0]

            i = 1
            for item in items:
                address, count, num = item
                rank.append({"id": i, "accountAddress": address, "dealAmount": num, "dealNum": count})
                i += 1

            resp = tools.dec_success_resp({"count": len(items), "rank": rank})
            return JsonResponse(resp, safe=False)

        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)


def get_block_miner_rank(request):
    if request.method == "GET":
        try:
            account = request.GET.get("account")
            from_time = request.GET.get('from_time')
            end_time = request.GET.get('end_time')

            if from_time and not end_time:
                resp = tools.dec_error_resp("end_time is null")
                return JsonResponse(resp)
            if not from_time and end_time:
                resp = tools.dec_error_resp("from_time is null")
                return JsonResponse(resp)

            rank = []
            addresses = get_account(account, from_time, end_time)
            if len(addresses) == 0:
                resp = tools.dec_success_resp({"count": 0, "rank": rank})
                return JsonResponse(resp, safe=False)
            accounts = Account.nodes.filter(address__in=addresses)

            # 账户
            transactions = []
            for a in accounts:
                next_nodes = a.transfer_out.all()
                for next_node in next_nodes:
                    transactions.append(next_node)

            # 交易
            blocks = []
            for t in transactions:
                next_block_nodes = t.trade.all()
                for next_node in next_block_nodes:
                    blocks.append(next_node)

            # 区块
            block_hashes = []
            while blocks:
                tmp_nodes = []
                for node in blocks:
                    block_hashes.append(node.hash)
                    next_nodes = node.next_block.all()
                    for next_node in next_nodes:
                        if next_node.id not in block_hashes:
                            tmp_nodes.append(next_node)
                blocks = tmp_nodes
            block_hashes = list(set(block_hashes))

            # 矿工
            param = {'hashes': block_hashes}
            query = 'match (m:Miner)-[p:PACK]->(b:Block) where b.hash in $hashes return m.address, count(p) order by count(p) desc'
            items = db.cypher_query(query, param)[0]
            i = 1
            for item in items:
                a, num = item
                rank.append({"id": i, "minerAddress": a, "packNum": num})

            resp = tools.dec_success_resp({"count": len(items), "rank": rank})
            return JsonResponse(resp, safe=False)
        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)
