# charset = utf-8

from django.http import JsonResponse
from neomodel import db

from domain.models import Label, RequestIp, gen_node_json, gen_node_json2, gen_relation_json2
from utils import tools


def get_label_list(request):
    if request.method == "GET":
        try:
            items = Label.nodes.all()
            labels = []

            i = 1
            for item in items:
                labels.append({"id": i, "name": item.label_name})
                i += 1

            resp = tools.dec_success_resp({"count": len(items), "labels": labels})
            return JsonResponse(resp, safe=False)
        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)


def get_request_ips(label, from_time, end_time):
    try:
        from_time = tools.from_str_to_time(from_time)
        end_time = tools.from_str_to_time(end_time)

        if label:
            if from_time:
                param = {"label": label, "from_time": from_time.timestamp(), "end_time": end_time.timestamp()}
                query = 'match (r:RequestIp)-[req:REQUEST]->(d:DomainName)-[]->(l:Label) ' \
                        'where l.label_name contains $label and req.date >= $from_time and req.date <= $end_time return r.request_ip limit 100'
            else:
                param = {"label": label}
                query = 'match (r:RequestIp)-[]->(d:DomainName)-[]->(l:Label) where l.label_name contains $label return r.request_ip'
            nodes = db.cypher_query(query, param)[0]
            ips = [node[0] for node in nodes]
        else:
            if from_time:
                param = {"from_time": from_time.timestamp(), "end_time": end_time.timestamp()}
                query = 'match (r:RequestIp)-[req:REQUEST]->(d:DomainName)-[]->(l:Label) ' \
                        'where req.date >= $from_time and req.date <= $end_time return r.request_ip limit 100'
                nodes = db.cypher_query(query, param)[0]
                ips = [node[0] for node in nodes]
            else:
                nodes = RequestIp.nodes[:100]
                ips = [node.request_ip for node in nodes]
        return ips
    except Exception as e:
        raise e


def get_domain_list(request):
    if request.method == "GET":
        try:
            label = request.GET.get("label")
            from_time = request.GET.get('from_time')
            end_time = request.GET.get('end_time')

            if from_time and not end_time:
                resp = tools.dec_error_resp("end_time is null")
                return JsonResponse(resp, safe=False)
            if not from_time and end_time:
                resp = tools.dec_error_resp("from_time is null")
                return JsonResponse(resp, safe=False)

            resp_nodes, resp_relations = [], []
            n_ids, r_ids = [], []
            ips = get_request_ips(label, from_time, end_time)
            if len(ips) == 0:
                resp = tools.dec_success_resp({"nodes": resp_nodes, "relations": resp_relations})
                return JsonResponse(resp, safe=False)

            param = {"ips": ips}
            query = 'match (r:RequestIp)-[req:REQUEST]->(d:DomainName)-[lab:LABEL]->(l:Label), (dns:DnsIp)-[par:PARSE]->(d)' \
                    'where r.request_ip in $ips return r, d, l, dns, req, lab, par'
            items = db.cypher_query(query, param)[0]
            for item in items:
                nodes = item[:4]
                relations = item[4:]
                for n in nodes:
                    if n.id in n_ids:
                        continue
                    resp_nodes.append(gen_node_json2(n))
                    n_ids.append(n.id)
                for r in relations:
                    if r.id in r_ids:
                        continue
                    resp_relations.append(gen_relation_json2(r))
                    r_ids.append(r.id)
            resp_nodes.sort(key=lambda n: n.get("id"))
            resp_relations.sort(key=lambda r: r.get("id"))

            resp = tools.dec_success_resp({"nodes": resp_nodes, "relations": resp_relations})
            return JsonResponse(resp, safe=False)
        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)


def get_label_rank(request):
    if request.method == "GET":
        try:
            label = request.GET.get("label")
            from_time = request.GET.get('from_time')
            end_time = request.GET.get('end_time')
            sort_type = request.GET.get("sort_type")

            if from_time and not end_time:
                resp = tools.dec_error_resp("end_time is null")
                return JsonResponse(resp, safe=False)
            if not from_time and end_time:
                resp = tools.dec_error_resp("from_time is null")
                return JsonResponse(resp, safe=False)

            rank = []
            ips = get_request_ips(label, from_time, end_time)
            param = {"ips": ips}
            if sort_type == 1:
                query = 'match (r:RequestIp)-[req:REQUEST]->(d:DomainName)-[lab:LABEL]->(l:Label) where r.request_ip in $ips ' \
                        'return l.label_name, count(distinct lab), count(req) order by count(distinct lab) desc'
            else:
                query = 'match (r:RequestIp)-[req:REQUEST]->(d:DomainName)-[lab:LABEL]->(l:Label) where r.request_ip in $ips ' \
                        'return l.label_name, count(distinct lab), count(req) order by count(req) desc'
            items = db.cypher_query(query, param)[0]
            i = 1
            for item in items:
                name, domain_num, req_count = item
                rank.append({"id": i, "name": name, "domainNum": domain_num, "requestNum": req_count})
                i += 1

            resp = tools.dec_success_resp({"count": len(items), "rank": rank})
            return JsonResponse(resp, safe=False)

        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)


def get_domain_rank(request):
    if request.method == "GET":
        try:
            label = request.GET.get("label")
            from_time = request.GET.get('from_time')
            end_time = request.GET.get('end_time')

            if from_time and not end_time:
                resp = tools.dec_error_resp("end_time is null")
                return JsonResponse(resp)
            if not from_time and end_time:
                resp = tools.dec_error_resp("from_time is null")
                return JsonResponse(resp)

            rank = []
            ips = get_request_ips(label, from_time, end_time)
            if len(ips) == 0:
                resp = tools.dec_success_resp({"count": 0, "rank": rank})
                return JsonResponse(resp, safe=False)

            param = {"ips": ips}
            query = 'match (r:RequestIp)-[]->(d:DomainName), (dns:DnsIp)-[p:PARSE]->(d) where r.request_ip in $ips ' \
                    'return d.domain_name, count(p) order by count(p) desc'
            items = db.cypher_query(query, param)[0]
            i = 1
            for item in items:
                name, parse_num = item
                rank.append({"id": i, "name": name, "parseNum": parse_num})
                i += 1

            resp = tools.dec_success_resp({"count": len(items), "rank": rank})
            return JsonResponse(resp, safe=False)
        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)


def get_request_ip_rank(request):
    if request.method == "GET":
        try:
            label = request.GET.get("label")
            from_time = request.GET.get('from_time')
            end_time = request.GET.get('end_time')

            if from_time and not end_time:
                resp = tools.dec_error_resp("end_time is null")
                return JsonResponse(resp)
            if not from_time and end_time:
                resp = tools.dec_error_resp("from_time is null")
                return JsonResponse(resp)

            rank = []
            ips = get_request_ips(label, from_time, end_time)
            if len(ips) == 0:
                resp = tools.dec_success_resp({"count": 0, "rank": rank})
                return JsonResponse(resp, safe=False)

            param = {"ips": ips}
            query = 'match (r:RequestIp)-[req:REQUEST]->(d:DomainName) where r.request_ip in $ips ' \
                    'return r.province, r.isp, count(req)'
            items = db.cypher_query(query, param)[0]
            i = 1
            for item in items:
                province, isp, request_num = item
                rank.append({"id": i, "province": province, "isp": isp, "requestNum": request_num})
                i += 1

            resp = tools.dec_success_resp({"count": len(items), "rank": rank})
            return JsonResponse(resp, safe=False)
        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)


def get_dns_ip_rank(request):
    if request.method == "GET":
        try:
            label = request.GET.get("label")
            from_time = request.GET.get('from_time')
            end_time = request.GET.get('end_time')

            if from_time and not end_time:
                resp = tools.dec_error_resp("end_time is null")
                return JsonResponse(resp)
            if not from_time and end_time:
                resp = tools.dec_error_resp("from_time is null")
                return JsonResponse(resp)

            rank = []
            ips = get_request_ips(label, from_time, end_time)

            if len(ips) == 0:
                resp = tools.dec_success_resp({"count": 0, "rank": rank})
                return JsonResponse(resp, safe=False)

            param = {"ips": ips}
            query = 'match (r:RequestIp)-[]->(d:DomainName), (dns:DnsIp)-[p:PARSE]->(d) where r.request_ip in $ips ' \
                    'return r.province,r.isp, count(distinct p) order by count(distinct p) desc'
            items = db.cypher_query(query, param)[0]
            i = 1
            for item in items:
                province, isp, parse_num = item
                rank.append({"id": i, "province": province, "isp": isp, "parseNum": parse_num})
                i += 1

            resp = tools.dec_success_resp({"count": len(items), "rank": rank})
            return JsonResponse(resp, safe=False)
        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)

