# charset = utf-8

from django.http import JsonResponse
from neomodel import Q, db

from route.models import FirstIp, EndIp, gen_node_json, gen_rel_json
from utils import tools


def get_route_param(request):
    if request.method == 'GET':
        try:
            province_city_dict = {}
            nodes = FirstIp.nodes.all()
            for node in nodes:
                if not province_city_dict.get(node.province):
                    province_city_dict[node.province] = []
                if not province_city_dict.get(node.city):
                    province_city_dict[node.province].append(node.city)

            provinces = []
            for province, _cities in province_city_dict.items():
                cities = []
                for c in _cities:
                    cities.append({"cityName": c})
                provinces.append({"provinceName": province, "cities": cities})

            root_ips = []
            nodes = EndIp.nodes.all()
            for node in nodes:
                root_ips.append(node.root_ip)
            root_ips = list(set(root_ips))

            resp = tools.dec_success_resp({"provinces": provinces, "rootIp": root_ips})
            return JsonResponse(resp, safe=False)
        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)


def get_first_ips(province, city, isp, root_ip, from_time, end_time):
    try:
        from_time = tools.from_str_to_time(from_time)
        end_time = tools.from_str_to_time(end_time)

        params = {}
        query = 'match (f:FirstIp)-[n:NEXT_IP]->() '
        query_extend = []
        if province:
            params['province'] = province
            query_extend.append('f.province = $province ')
        if city:
            params['city'] = city
            query_extend.append('f.city = $city ')
        if isp:
            params['isp'] = isp
            query_extend.append('f.isp = $isp ')
        if root_ip:
            params['root_ip'] = root_ip
            query_extend.append('f.root_ip = $root_ip ')
        if from_time and end_time:
            params['from_time'] = from_time.timestamp()
            params['end_time'] = end_time.timestamp()
            query_extend.append('n.date >= $from_time ')
            query_extend.append('n.date <= $end_time ')
        if len(query_extend) != 0:
            query += "where "
            for i in range(len(query_extend)):
                if i == 0:
                    query += query_extend[i]
                else:
                    query += 'and ' + query_extend[i]
        query += 'return f.ip limit 100'

        items = db.cypher_query(query, params)[0]
        ips = [item[0] for item in items]
        return ips
    except Exception as e:
        raise e


def get_route_list(request):
    if request.method == 'GET':
        try:
            province = request.GET.get('province')
            city = request.GET.get('city')
            isp = request.GET.get('isp')
            root_ip = request.GET.get('root_ip')
            from_time = request.GET.get('from_time')
            end_time = request.GET.get('end_time')

            if from_time and not end_time:
                resp = tools.dec_error_resp("end_time is null")
                return JsonResponse(resp, safe=False)
            if not from_time and end_time:
                resp = tools.dec_error_resp("from_time is null")
                return JsonResponse(resp, safe=False)

            resp_nodes, resp_relations = [], []
            ips = get_first_ips(province, city, isp, root_ip, from_time, end_time)  # 当前遍历到的节点集合

            if len(ips) == 0:
                resp = tools.dec_success_resp({"nodes": resp_nodes, "relations": resp_relations})
                return JsonResponse(resp, safe=False)

            current_nodes = FirstIp.nodes.filter(ip__in=ips).all()
            while current_nodes:
                tmp_nodes = []
                tmp_node_ids = []  # 手动去掉重复的node
                for node in current_nodes:
                    resp_nodes.append(gen_node_json(node))
                    next_nodes = node.next_ip.all()
                    for next_node in next_nodes:
                        if next_node.id not in tmp_node_ids:
                            tmp_nodes.append(next_node)
                            for r in node.next_ip.all_relationships(next_node):
                                resp_relations.append(gen_rel_json(r))
                        tmp_node_ids.append(next_node.id)
                current_nodes = tmp_nodes

            resp = tools.dec_success_resp({"nodes": resp_nodes, "relations": resp_relations})
            return JsonResponse(resp, safe=False)
        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)


def get_route_rank(request):
    if request.method == 'GET':
        try:
            province = request.GET.get('province')
            city = request.GET.get('city')
            isp = request.GET.get('isp')
            root_ip = request.GET.get('root_ip')
            from_time = request.GET.get('from_time')
            end_time = request.GET.get('end_time')

            if from_time and not end_time:
                resp = tools.dec_error_resp("end_time is null")
                return JsonResponse(resp, safe=False)
            if not from_time and end_time:
                resp = tools.dec_error_resp("from_time is null")
                return JsonResponse(resp, safe=False)

            rank = []
            first_node_ips = get_first_ips(province, city, isp, root_ip, from_time, end_time)
            if len(first_node_ips) == 0:
                resp = tools.dec_success_resp({"count": 0, "rank": rank})
                return JsonResponse(resp, safe=False)

            param = {'start_ips': first_node_ips}
            query = 'match (n:RouteIp) where n.start_ip in $start_ips with n, size((n)-[]-()) as size order by size desc limit 100 return n.ip, n.province ,n.isp, size'
            items = db.cypher_query(query, param)[0]

            for item in items:
                if len(item) != 0:
                    rank.append({"ip": item[0], "province": item[1], "isp": item[2], "degree": item[3]})

            resp = tools.dec_success_resp({"count": len(items), "rank": rank})
            return JsonResponse(resp, safe=False)
        except Exception as e:
            resp = tools.dec_error_resp(e)
            return JsonResponse(resp, safe=False)

